try:
    from IPython import get_ipython

    d = get_ipython()

    if d and "IPKernelApp" in d.config:  # Checks for Jupyter kernel

        from rich.jupyter import JUPYTER_HTML_FORMAT

        if "margin:0px" not in JUPYTER_HTML_FORMAT:
            print(
                'mrich: Jupyter may show unwanted line-spacing, run `"python -c import mrich; mrich.patch_rich_jupyter_margins()"` to patch'
            )
except:
    pass


def patch_rich_jupyter_margins():

    from rich.jupyter import JUPYTER_HTML_FORMAT

    if "margin:0px" in JUPYTER_HTML_FORMAT:
        print("mrich: rich.jupyter already patched")
        return

    from rich import jupyter

    RICH_JUPYTER_PATH = jupyter.__file__

    # Define the strings to search and replace
    target_string = """<pre style="white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace">{code}</pre>"""
    replacement_string = """<pre style="white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace;margin:0px">{code}</pre>"""

    # Step 1: Read the file content
    with open(RICH_JUPYTER_PATH, "r") as file:
        file_contents = file.read()

    # Step 2: Replace the target string with the replacement string
    updated_contents = file_contents.replace(target_string, replacement_string)

    # Step 3: Write the modified content back to the file
    with open(RICH_JUPYTER_PATH, "w") as file:
        file.write(updated_contents)

    print("mrich: patched rich.jupyter HTML margins")



from .console import console, console_print

dict_keys = type(dict().keys())


# wrappers
def print(*args, **kwargs):

    new_args = []

    for i, arg in enumerate(args):

        try:
            from pandas import DataFrame, Index

            if isinstance(arg, DataFrame):
                from .df import df_to_table

                # console_print(arg, type(arg))
                table = df_to_table(arg)
                new_args.append(table)
                continue

            elif isinstance(arg, Index):
                new_args.append(set(arg))
                continue

        except ModuleNotFoundError as e:
            pass
        except Exception as e:
            raise

        if isinstance(arg, dict_keys):
            new_args.append(set(arg))

        else:
            new_args.append(arg)

    console_print(*new_args, **kwargs)


def out(*args, **kwargs):
    console.out(*args, **kwargs)


def rule(*args, **kwargs):
    console.rule(*args, **kwargs)


### PROGRESS

CURRENT_PROGRESS = None


def clear_progress():
    global CURRENT_PROGRESS
    CURRENT_PROGRESS = None


def track(*args, prefix: str = "Working...", **kwargs):

    from .colors import THEME
    from rich.console import Console
    from rich.progress import (
        Progress,
        TextColumn,
        BarColumn,
        TaskProgressColumn,
        TimeRemainingColumn,
        TimeElapsedColumn,
    )

    global CURRENT_PROGRESS

    console = Console(theme=THEME)

    CURRENT_PROGRESS = Progress(
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TaskProgressColumn(),
        TimeElapsedColumn(),
        TimeRemainingColumn(),
        TextColumn("{task.fields}"),
        # TextColumn("{task.fields[suffix]}"),
        console=console,
    )

    # CURRENT_PROGRESS.tasks[0].fields["suffix"] = ""

    with CURRENT_PROGRESS:
        yield from CURRENT_PROGRESS.track(*args, description=prefix, **kwargs)


def set_progress_field(key: str = None, value=None, **kwargs):
    progress = CURRENT_PROGRESS

    if not progress:
        return

    task = progress.tasks[0]

    if kwargs:
        for key, value in kwargs.items():
            task.fields[key] = value
    else:
        task.fields[key] = value


def set_progress_prefix(text):
    progress = CURRENT_PROGRESS

    if not progress:
        return

    task = progress.tasks[0]
    task.description = text


# def set_progress_suffix(text):
#     set_progress_field("suffix", text)
# task = progress.tasks[0]
# task.description = text



import re


def detect_format_prefix(message):

    match = re.match(r"^\[.*?\]", message)

    if match:
        match = match.group()

    return match


def strip_formats(*messages, text="", separator=" "):
    formats = []
    start = len(text) + 1
    for message in messages:

        if hasattr(message, "__rich__"):
            message = message.__rich__()
        else:
            message = str(message)

        prefix = detect_format_prefix(message)

        prefix_found = prefix is not None

        if prefix_found:
            message = message.removeprefix(prefix)
            prefix = prefix.removeprefix("[").removesuffix("]")

        end = start + len(message)
        text += separator + message

        if prefix_found:
            formats.append((prefix, start, end + 2))

        start = end

    return text, formats



from setuptools import setup, find_packages

setup(
    name="mrich",
    version="1.0",
    author="Max Winokan",
    author_email="mwinokan@me.com",
    description="Pretty-printing wrappers around Rich",
    url="https://github.com/mwinokan/mrich",
    packages=find_packages(),
    python_requires=">=3.8",
    install_requires=[
        "rich",
    ],
)



def test_wrappers():

    import mrich

    # large header banner
    mrich.h1("Welcome to mrich")

    # Regular & rich text printing
    mrich.print(
        "mrich offers a simple print wrapper which plays nicely with live rich elements"
    )
    mrich.print("Emphasis can be placed with different mrich functions:")
    mrich.bold("bold text")
    mrich.italic("italic text")
    mrich.underline("underlined text")
    mrich.print(
        "rich markup is also [bold green]supported[reset] via square-bracket syntax"
    )

    # smaller header banner
    mrich.h2("Event logging function")

    # Successes, errors, and warnings
    mrich.success("Success statements are the most eye-catching to the user")
    mrich.error("Error statements are also highlighted")
    mrich.warning("As are warnings")

    # Other pre-made styles:
    mrich.debug("Inobtrusive debug statements")
    mrich.prompt("Eye-catching prompt")
    mrich.reading("this_file_is_being_read.txt")
    mrich.writing("this_file_is_being_written.txt")
    mrich.disk("this_file_is_being_munged.txt", prefix="Modifying")

    # Format variables:
    mrich.var("variable", "value")
    mrich.var("#samples", 123)
    mrich.var("frequency", 12.7, "GHz")
    mrich.var("file", "/example/path/file.html")
    mrich.var("green number", 456, separator=":", color="green")

    # smaller section separator/header
    mrich.h3("This is a smaller header panel")

    # dynamic elements

    import time

    with mrich.clock("Waiting for something"):
        time.sleep(1)

    with mrich.loading("Loading"):
        time.sleep(1)
        mrich.print("Interruptions don't disrupt live elements")
        time.sleep(2)

    with mrich.spinner("Spinning"):
        time.sleep(1)

    for i in mrich.track(range(20), prefix="tracking progress", total=20):
        time.sleep(0.2)
        if i == 9:
            mrich.print("halfway there!")
            mrich.set_progress_field("i_halfway", i)

    # tabular data
    import pandas as pd

    df = pd.DataFrame(
        [
            dict(a=1, b=2),
            dict(a=2, b=3),
            dict(a=3, b=4),
            dict(a=4, b=5),
        ]
    )
    mrich.print("This is a dataframe printed as a rich.Table", df)

    raise ValueError("rich formatted traceback is enabled by default")


if __name__ == "__main__":
    test_wrappers()
